<table id="CVEs" class="table table-hover table-striped">
  <thead>
    <tr class="warning">
      <td></td><td>ID</td><td>CVSS</td><td>Summary</td><td>Last (major) update</td><td>Published</td>
    </tr>
  </thead>
  <tbody>
    {% for c in cve %}
    {% if 'whitelisted' in c %}{%set class="whitelisted"%}{% elif 'blacklisted' in c %}{%set class="blacklisted"%}
    {% else %}{%set class=""%}{% endif %}
      <tr id ="{{ c['id'] }}" class="{{class}}" {{'style=color:'+c['color'] if 'color' in c else ''}}>
        <td>
          {% if 'icon' in c %}
            <span class="glyphicon glyphicon-{{c['icon']}}"></span>
          {% else %}
            {% if 'whitelisted' in c %}
              <span class="glyphicon glyphicon-bookmark"></span>
            {% elif 'blacklisted' in c %}
              <span class="glyphicon glyphicon-eye-close"></span>
            {% endif %}
          {% endif %}
        </td>
        <td>
          <a href="/cve/{{ c['id'] }}">{{ c['id'] }}</a>
          {% if 'reason' in c %}
            <br /><span class="badge">{{c['reason']}}</span>
          {% endif %}
        </td>
        <td>
          {{ c['cvss'] }}
        </td>
        <td>
          <div rel="tooltip" title="{{ c['summary'] }}">
            {{ c['summary'][:250] }}
          </div>
        </td>
        <td>
          {{ c['Modified'].strftime('%d-%m-%Y - %H:%M') }}
        </td>
        <td>
          {{ c['Published'].strftime('%d-%m-%Y - %H:%M') }}
        </td>
      </tr>
    {% endfor %}
  </tbody>
</table>
<a href="#" class="back-to-top">Back to Top</a>
<a href="#" class="select-submit">Mark selected</a>
