.whitelisted{
    color: #AA0000;
}
.blacklisted{
    color: #555588;
}
.table.table-striped tr.selected{
    background-color: #556677;
    color: #FFFFFF;
}
.listInput{
    width:calc(50% - 25px);
    min-width:200px;
}
.listInput textarea, input[type=text]{
    width:100%;
}
.listInput input[type=submit]{
    margin:0px auto;
    width:150px;
}
.listInputCenter p{
    padding-top:100px;
    text-align:center;
}
.invisiTable td{
    padding:0px 5px;
}
.badge a{
    color:white;
}
#admin{
    float: right;
}
ul.nav li.dropdown:hover > ul.dropdown-menu {
    display: block;
}
.back-to-top {
    position: fixed;
    bottom: 2em;
    right: 0px;
    text-decoration: none;
    color: #000000;
    background-color: rgba(235, 180, 180, 0.70);
    font-size: 12px;
    padding: 1em;
    display: none;
}
.select-submit {
    position: fixed;
    bottom: 6em;
    right: 0px;
    text-decoration: none;
    color: #000000;
    background-color: rgba(180, 180, 235, 0.70);
    font-size: 12px;
    padding: 1em;
    display: none;
}

.back-to-top:hover {    
    background-color: rgba(135, 135, 135, 0.50);
}

.table-nonfluid {
   width: auto;
}

.block {
   display: inline-block;
}
.semiCollapsed {
   overflow: hidden;
   text-overflow: ellipsis;
   max-height:110px;
}

.pagerdiv {
   width:100%;
   text-align:center;
}

#status {
   position:fixed;
   top: 50%;
   left: 50%;
   width:50em;
   height:6em;
   margin-top: -3em;
   margin-left: -25em;
   border: 1px solid #ccc;
   text-align:center;
}

.hidden {
   visibility:hide;
}

ul.via4 {
   padding-left:0px;
}

ul.via4 > li {
  margin: 3px 0px;
  padding-left: 3px;
  list-style-type: none;
  position: relative;
}

ul.via4 > li:after {
  padding:0;margin:0;display:block;/* not really needed */
  content: "";
  height:90%;
  width:3px;
  background-color: #8AE790;
  position: absolute;
  left:0;
  top:5%;
}

